/******************************************************************************
*  List of available character classes (singleton pattern).
*
*  @author   Daniel R. Collins (dcollins@superdan.net)
*  @since    2014-05-22
*  @version  1.0
******************************************************************************/

import java.util.ArrayList;

public class ClassList {
	//--------------------------------------------------------------------------
	//  Fields
	//--------------------------------------------------------------------------
	static ClassList instance = null;
	ArrayList<ClassInfo> list;

	//--------------------------------------------------------------------------
	//  Constructors
	//--------------------------------------------------------------------------

	/**
	*  Constructor.
	*/
	protected ClassList () {
		list = new ArrayList<ClassInfo>();
		list.add(new ClassInfo(ClassID.Fighter, "Ftr", 2000, 9, 8, 2, 1.0));
		list.add(new ClassInfo(ClassID.Thief, "Thf", 1250, 10, 6, 1, 2.0/3));
		list.add(new ClassInfo(ClassID.Wizard, "Wiz", 2500, 11, 4, 1, 0.5));
	}

	//--------------------------------------------------------------------------
	//  Methods
	//--------------------------------------------------------------------------
	public int size() { return list.size(); }
	public ClassInfo get (int index) { return list.get(index); }

	/**
	*  Accesses the singleton class instance.
	*/
	public static ClassList getInstance() {
		if (instance == null) {
			instance = new ClassList();		
		}
		return instance;	
	}

	/**
	*  Returns the ClassInfo for a specified ClassID.
	*/
	public static ClassInfo getInfo (ClassID classID) {
		ClassList cl = getInstance();
		for (int i = 0; i < cl.size(); i++) {
			ClassInfo classInfo = cl.get(i);
			if (classID == classInfo.getClassID())
				return classInfo;
		}
		return null;
	}

	/**
	*  Returns a string representation of this class info.
	*/
	public String toString() {
		return this.toString();
	}	

	/**
	*  Main test method.
	*/
	public static void main (String[] args) {
		ClassList c = ClassList.getInstance();
		for (int i = 0; i < c.size(); i++) {
			System.out.println(c.get(i));
		}
		System.out.println(ClassList.getInfo(ClassID.Thief));
	}
}

