/******************************************************************************
*  Record of a class gained by a persona (XP, level, hit points, etc.).
*
*  @author   Daniel R. Collins (dcollins@superdan.net)
*  @since    2014-05-22
*  @version  1.0
******************************************************************************/

public class ClassRecord {
	//--------------------------------------------------------------------------
	//  Fields
	//--------------------------------------------------------------------------

	/** Class ID/name for this class. */
	ClassID classID;

	/** Level in this class. */
	int level;

	/** Hit points earned for this class. */
	int hitPoints;

	/** Experience points earned towards this class. */
	int XP;

	//--------------------------------------------------------------------------
	//  Constructors
	//--------------------------------------------------------------------------

	/**
	*  Constructor (1st level, random hit points).
	*/
	public ClassRecord (ClassID classID, int hpBonus) {
		this(classID, 1, 1, 0);
		hitPoints = getInfo().hitDiceForLevel(1).rollPlus(hpBonus);
	}

	/**
	*  Constructor (full fields).
	*/
	public ClassRecord (ClassID classID, 
			int level, int hitPoints, int XP) {
		this.classID = classID;
		this.level = level;
		this.hitPoints = hitPoints;
		this.XP = XP;
	}

	//--------------------------------------------------------------------------
	//  Methods
	//--------------------------------------------------------------------------
	public ClassID getClassID () { return classID; }
	public int getLevel () { return level; }
	public int getHitPoints () { return hitPoints; }
	public int getXP () { return XP; }

	/**
	*  Returns the associated class info.
	*/
	public ClassInfo getInfo () {
		return ClassList.getInfo(classID);
	}

	/**
	*  Returns the base attack bonus for this class/level.
	*/
	public int baseAttackBonus () {
		return getInfo().atkBonusForLevel(level);
	}

	/**
	*  Returns the total base hit dice for this class/level.
	*/
	public Dice baseHitDice () {
		return getInfo().hitDiceTotalForLevel(level);
	}

	/**
	*  Add XP to this class & handle level-ups.
	*  TODO: Having the Con bonus in here is pretty clunky.
	*/
	public void addXP (int awardXP, int hpBonus) {
		int newTotal = XP + awardXP;
		int newLevel = getInfo().levelForXP(newTotal);

		// Limit to one level per award
		if (newLevel > level + 1) {
			newLevel = level + 1;
			newTotal = getInfo().XPForLevel(newLevel+1)-1;
		}
		XP = newTotal;

		// Update hit points
		if (newLevel > level) {
			level = newLevel;
			Dice newDice = getInfo().hitDiceForLevel(newLevel);
			if (newDice.getNum() > 0)
				hitPoints += newDice.rollPlus(hpBonus);
			else // Constant hp add only
				hitPoints += newDice.roll();
		}
	}

	/**
	*  Returns a string representation of this class record.
	*/
	public String toString() {
		return classID + "(" + level + ", " + hitPoints + ", " + XP + ")";
	}	

	/**
	*  Main test method.
	*/
	public static void main (String[] args) {
	}

	/*--------------------------------------------------------------------------
	* NOTES:
	* TODO: Potentially could drop the ClassID and just link to the ClassInfo.
	*-------------------------------------------------------------------------*/
}

