/******************************************************************************
*  Creature (hostile or benign).
*
*  @author   Daniel R. Collins (dcollins@superdan.net)
*  @since    2014-05-20
*  @version  1.0
******************************************************************************/

public class Creature implements Comparable<Creature> {
	//--------------------------------------------------------------------------
	//  Fields
	//--------------------------------------------------------------------------
	String name, race;
	Alignment alignment;
	int armorClass, moveInches;
	Dice hitDice;
	int hitPoints, maxHitPoints;
	Attack attack;

	//--------------------------------------------------------------------------
	//  Constructors
	//--------------------------------------------------------------------------

	/**
	*  Constructor (primitive stats).
	*/
	public Creature (int AC, int MV, int HD) {
		this(null, null, Alignment.Neutral, AC, MV,
			new Dice(HD, 6), new Attack(HD, 1));
	}

	/**
	*  Constructor (primitive stats, attack rate, damage dice num).
	*/
	public Creature (int AC, int MV, int HD, int atkRate, int damDice) {
		this(null, null, Alignment.Neutral, AC, MV,
			new Dice(HD, 6), new Attack(HD, atkRate, damDice));
	}

	/**
	*  Constructor (full fields).
	*/
	public Creature (String name, String race, Alignment alignment, 
			int AC, int MV, Dice hitDice, Attack attack) {
		this.name = name;
		this.race = race;
		this.alignment = alignment;
		this.armorClass = AC;
		this.moveInches = MV;
		this.hitDice = hitDice;
		this.attack = attack;
		rollHitPoints();
	}

	//--------------------------------------------------------------------------
	//  Methods
	//--------------------------------------------------------------------------
	public String getName() { return name; }
	public String getRace() { return race; }
	public Alignment getAlignment() { return alignment; }
	public int getArmorClass() { return armorClass; }
	public int getMoveInches() { return moveInches; }
	public int getHitDiceNum() { return hitDice.getNum(); }
	public int getHitPoints() { return hitPoints; }
	public int getMaxHitPoints() { return maxHitPoints; }
	public int getAC() { return getArmorClass(); }
	public int getHD() { return getHitDiceNum(); }
	public int getHP() { return getHitPoints(); }
	public int getMV() { return getMoveInches(); }

	/**
	*  Compares to another creature by hit dice num.
	*/
   public int compareTo(Creature other) {
		int myHD = getHitDiceNum();
		int otherHD = other.getHitDiceNum();
		if (myHD > otherHD) 
			return 1;
		else if (myHD < otherHD)
			return -1;
		else
			return 0;
    }

	/**
	*  Rolls hit points from hit dice.
	*/
	private void rollHitPoints() {
		maxHitPoints = hitDice.roll();
		hitPoints = maxHitPoints;
	}

	/**
	*  Takes damage (minimum 0 hp).
	*/
	public void takeDamage (int damage) {
		hitPoints -= damage;
		if (hitPoints < 0)
			hitPoints = 0;
	}

	/**
	*  Checks if the creature is dead.
	*/
	public boolean isDead() {
		return hitPoints <= 0;	
	
	}

	/**
	*  Sets hit points to maxHitPoints.
	*/
	public void setHitPointsToMax () {
		hitPoints = getMaxHitPoints();
	}

	/**
	*  Returns a string representation of this creature.
	*/
	public String toString() {
		return (name == null ? "Creature" : name)
		  + (race == null ? "" : ", " + race)
        + " (AC " + armorClass + ", MV " + moveInches 
		  + ", HD " + hitDice.getNum() + ", hp " + hitPoints + ")";
		// TODO: Add primitive Atk, Dam info.
	}	

	/**
	*  Main test method.
	*/
	public static void main (String[] args) {
		Creature c = new Creature(5, 9, 4);
		System.out.println(c);
	}

	/*--------------------------------------------------------------------------
	* TODO (at some point):
	* numberAppearing (dice & multiplier), inLairPct, treasureType (char, etc.)
	* moveFly, moveSwim (etc.?)
	* Multiple attack forms (in ArrayList, possibly persona can register/swap)
	* Method "spawn" to create copies of one monster (with new hp)	
	* Fractional hit dice (e.g., rat, centipede, OD&D skeleton).
	*-------------------------------------------------------------------------*/
}

