/******************************************************************************
*  Dice group for random rolls.
*
*  @author   Daniel R. Collins (dcollins@superdan.net)
*  @since    2014-05-20
*  @version  1.0
******************************************************************************/

public class Dice {
	//--------------------------------------------------------------------------
	//  Fields
	//--------------------------------------------------------------------------

	/** Number of dice. */
	private int num;
	
	/** Sides on the dice. */
	private int sides;
	
	/** Modifier added to the dice. */
	private int modifier;

	//--------------------------------------------------------------------------
	//  Constructors
	//--------------------------------------------------------------------------

	/**	
	*  Constructor (single die).
	*/
	Dice (int sides) {
		this(1, sides, 0);
	}

	/**	
	*  Constructor (no modifier).
	*/
	Dice (int num, int sides) {
		this(num, sides, 0);
	}

	/**	
	*  Constructor (full fields).
	*/
	Dice (int num, int sides, int mod) {
		this.num = num;
		this.sides = sides;
		this.modifier = mod;	
	}

	//--------------------------------------------------------------------------
	//  Methods
	//--------------------------------------------------------------------------
	public int getNum() { return num; }
	public int getSides() { return sides; }
	public int getModifier() { return modifier; }

	/**	
	*  Rolls the dice (returns at least 1).
	*/
	public int roll () {
		int sum = modifier;
		for (int i = 0; i < num; i++) {
			sum += (int)(Math.random() * sides) + 1;
		}
		return (sum > 0 ? sum : 1);
	}

	/**	
	*  Rolls the dice with added modifier (returns at least 1).
	*/
	public int rollPlus (int plus) {
		int sum = modifier + plus;
		for (int i = 0; i < num; i++) {
			sum += (int)(Math.random() * sides) + 1;
		}
		return (sum > 0 ? sum : 1);
	}

	/**
	*  Returns a string representation of this dice group.
	*/
	public String toString() {
		String s = num + "d" + sides;
		if (modifier < 0)
			s += modifier;
		else if (modifier > 0)
			s += "+" + modifier;
		return s;			
	}
	
	/**
	*  Main test function.
	*/
	public static void main (String[] args) {
		System.out.println("Roll 1d6: " + (new Dice(6)).roll());
		Dice dice = new Dice(2, 6);
		System.out.println("Roll " + dice + ": " + dice.roll());
		dice = new Dice(4, 6, 1);
		System.out.println("Roll " + dice + ": " + dice.roll());
		dice = new Dice(1, 6, -7);
		System.out.println("Roll " + dice + ": " + dice.roll());
	}
}

