/******************************************************************************
*  Statistical record for a given persona level.
*
*  @author   Daniel R. Collins (dcollins@superdan.net)
*  @since    2014-05-23
*  @version  1.0
******************************************************************************/

public class LevelStats {
	//--------------------------------------------------------------------------
	//  Fields
	//--------------------------------------------------------------------------

	/** Number of characters summed. */
	int num;

	/** Sum of hit points. */
	long sumHitPoints;

	/** Sum square of hit points. */
	long sumSquareHitPoints;

	/** Sum of the six ability scores. */
	long [] sumAbilityScore;

	/** Sum square of the six ability scores. */
	long [] sumSquareAbilityScore;

	//--------------------------------------------------------------------------
	//  Constructors
	//--------------------------------------------------------------------------

	/**
	*  Constructor.
	*/
	public LevelStats () {
		num = 0;
		sumHitPoints = 0;
		sumSquareHitPoints = 0;
		sumAbilityScore = new long[Ability.values().length];
		sumSquareAbilityScore = new long[Ability.values().length];
	}

	//--------------------------------------------------------------------------
	//  Methods
	//--------------------------------------------------------------------------

	/**
	*  Get the total number of personas accounted.
	*/
	public int getNum () { return num; }

	/**
	*  Square an integer.
	*/
	static long square (long i) {
		return i * i;	
	}

	/**
	*  Compute a mean from a sum.
	*/
	int mean (long sum) {
		return (int)(Math.round((double)sum/num));	
	}

	/**
	*  Compute a standard deviation from a sum and sum-of-squares.
	*/
	int stdev (long sum, long sumSquare) {
		return (int) Math.round(
			Math.sqrt((sumSquare - (double)square(sum)/num)/num));
	}

	/**
	*  Add another persona into these stats.
	*/
	public void addPersona (Persona p) {
		num++;
		sumHitPoints += p.getMaxHitPoints();
		sumSquareHitPoints += square(p.getMaxHitPoints());
 		for (int i = 0; i < sumAbilityScore.length; i++) {
 			sumAbilityScore[i] += p.getScore(i);
 			sumSquareAbilityScore[i] += square(p.getScore(i));
 		}
	}

	/**
	*  Generate & print all the stats we have available.
	*/
	public void printAllStats (boolean printStdev) {
		System.out.print(num + " ");
		if (num < 1) {
			System.out.println("No data");
			return;
		}
		System.out.print(mean(sumHitPoints) + " ");
		if (printStdev)
			System.out.print(stdev(sumHitPoints, sumSquareHitPoints) + " ");
 		for (int i = 0; i < sumAbilityScore.length; i++) {
			System.out.print(mean(sumAbilityScore[i]) + " ");
			if (printStdev) {
				System.out.print(stdev(sumAbilityScore[i], 
					sumSquareAbilityScore[i]) + " ");
			}
 		}
		System.out.println("");
	}

	/*--------------------------------------------------------------------------
	* NOTES:
	*-------------------------------------------------------------------------*/
}

